@@ -215,6 +215,9 @@ class OMNI2IR(GenericVisitor):
         if module is not None:
             # Import symbol attributes from module, if available
             for k, v in module.symbol_attrs.items():
+                # Don't import private module symbols
+                if v.private or (module.default_access_spec == "private" and not v.public):
+                    continue
                 if k in rename_list:
                     local_name = rename_list[k].name
                     scope.symbol_attrs[local_name] = v.clone(imported=True, module=module, use_name=k)
@@ -462,8 +465,9 @@ class OMNI2IR(GenericVisitor):
             if module_type and module_type.dtype.module != BasicType.DEFERRED:
                 return module_type.dtype.module
 
-        return Module(name=name, parent=scope)
-
+        module = Module(name=name, parent=scope)
+        self.definitions[name] = module
+        return module
 
     def visit_FmoduleDefinition(self, o, **kwargs):
         # Update the symbol map with local entries
