@@ -15,7 +15,6 @@ language features.
 # pylint: disable=too-many-lines
 
 from pathlib import Path
-from shutil import rmtree
 from time import perf_counter
 import numpy as np
 import pytest
@@ -23,7 +22,7 @@ import pytest
 from loki import (
     Module, Subroutine, FindVariables, BasicType, config, Sourcefile,
     RawSource, RegexParserClass, ProcedureType, DerivedType,
-    PreprocessorDirective, config_override, gettempdir
+    PreprocessorDirective, config_override
 )
 from loki.build import jit_compile, clean_test
 from loki.expression import symbols as sym
@@ -1852,7 +1851,7 @@ end subroutine test_inline_comments
 
 @pytest.mark.parametrize('from_file', (True, False))
 @pytest.mark.parametrize('preprocess', (True, False))
-def test_source_sanitize_fp_source(from_file, preprocess):
+def test_source_sanitize_fp_source(tmp_path, from_file, preprocess):
     """
     Test that source sanitizing works as expected and postprocessing
     rules are correctly applied
@@ -1871,11 +1870,7 @@ end subroutine some_routine
 """.strip()
 
     if from_file:
-        workdir = gettempdir()/'test_source_sanitize_fp_source'
-        if workdir.exists():
-            rmtree(workdir)
-        workdir.mkdir()
-        filepath = workdir/'some_routine.F90'
+        filepath = tmp_path/'some_routine.F90'
         filepath.write_text(fcode)
         obj = Sourcefile.from_file(filepath, frontend=FP, preprocess=preprocess, defines=('MY_VAR=5',))
     else:
@@ -1892,9 +1887,6 @@ end subroutine some_routine
 
     assert 'newunit=fu' in obj.to_fortran()
 
-    if from_file:
-        rmtree(workdir)
-
 
 @pytest.mark.parametrize('preprocess', (True, False))
 def test_source_sanitize_fp_subroutine(preprocess):
@@ -2031,3 +2023,46 @@ end subroutine routine_variables_dimensions
     assert isinstance(routine.variable_map['v6'].shape[0], sym.Quotient)
     assert isinstance(routine.variable_map['v6'].shape[1], sym.Power)
     assert isinstance(routine.variable_map['v6'].shape[2], sym.Quotient)
+
+@pytest.mark.parametrize('frontend', available_frontends())
+def test_import_of_private_symbols(here, frontend):
+    """
+    Verify that only public symbols are imported from other modules.
+    """
+    code_mod_private = """
+module mod_private
+    private
+    integer :: var
+end module mod_private
+    """
+    code_mod_public = """
+module mod_public
+    public
+    integer:: var
+end module mod_public
+    """
+    code_mod_main = """
+module mod_main
+    use mod_public
+    use mod_private
+contains
+
+    subroutine test_routine()
+        integer :: result
+        result = var
+    end subroutine test_routine
+
+end module mod_main
+    """
+
+    mod_private = Module.from_source(code_mod_private, frontend=frontend)
+    mod_public = Module.from_source(code_mod_public, frontend=frontend)
+    mod_main = Module.from_source(code_mod_main, frontend=frontend, definitions=[mod_private, mod_public])
+    var = mod_main.subroutines[0].body.body[0].rhs
+    # Check if this is really our symbol
+    assert var.name == "var"
+    assert var.scope is mod_main
+    # Check if the symbol is imported
+    assert var.type.imported is True
+    # Check if the symbol comes from the mod_public module
+    assert var.type.module is mod_public
