@@ -63,14 +63,14 @@ class Node:
     metadata.
 
     Attributes
-    -----------
+    ----------
     traversable : list of str
         The traversable fields of the Node; that is, fields walked over by
         a :any:`Visitor`. All arguments in :py:meth:`__init__` whose
         name appear in this list are treated as traversable fields.
 
     Parameters
-    -----------
+    ----------
     source : :any:`Source`, optional
         the information about the original source for the Node.
     label : str, optional
@@ -1535,7 +1535,10 @@ class TypeDef(ScopedNode, InternalNode, _TypeDefBase):
             return None
         if not self.parent:
             return BasicType.DEFERRED
-        return self.parent.symbol_attrs[self.extends].dtype.typedef
+        parent_type = self.parent.symbol_attrs.lookup(self.extends)
+        if not (parent_type and isinstance(parent_type.dtype, DerivedType)):
+            return BasicType.DEFERRED
+        return parent_type.dtype.typedef
 
     @property
     def declarations(self):
