@@ -21,7 +21,7 @@ from loki import (
     StringSubscript, Conditional, CallStatement, ProcedureSymbol,
     FindVariables
 )
-from loki.build import jit_compile, jit_compile_lib, clean_test
+from loki.build import jit_compile, jit_compile_lib, clean_test, Obj
 from loki.frontend import available_frontends, OMNI, OFP
 
 
@@ -32,7 +32,8 @@ def fixture_here():
 
 @pytest.fixture(scope='module', name='builder')
 def fixture_builder(here):
-    return Builder(source_dirs=here, build_dir=here/'build')
+    yield Builder(source_dirs=here, build_dir=here/'build')
+    Obj.clear_cache()
 
 
 @pytest.mark.parametrize('frontend', available_frontends())
@@ -1519,3 +1520,42 @@ end module some_mod
     assert not extended_extended_type.imported_symbol_map
     #check for non-empty declarations
     assert all(decl.symbols for decl in extended_extended_type.declarations)
+
+
+@pytest.mark.parametrize('frontend', available_frontends())
+@pytest.mark.parametrize('qualified_import', (True, False))
+def test_derived_type_inheritance_missing_parent(frontend, qualified_import, tmp_path):
+    fcode_parent = """
+module parent_mod
+    implicit none
+    type, abstract, public :: parent_type
+        integer :: val
+    end type parent_type
+end module parent_mod
+    """.strip()
+
+    fcode_derived = f"""
+module derived_mod
+    use parent_mod{", only: parent_type" if qualified_import else ""}
+    implicit none
+    type, public, extends(parent_type) :: derived_type
+        integer :: val2
+    end type derived_type
+contains
+    subroutine do_something(this)
+        class(derived_type), intent(inout) :: this
+        this%val = 1
+        this%val2 = 2
+    end subroutine do_something
+end module derived_mod
+    """.strip()
+
+    parent = Module.from_source(fcode_parent, frontend=frontend, xmods=[tmp_path])
+
+    # Without enrichment we obtain only DEFERRED type information (but don't fail!)
+    derived = Module.from_source(fcode_derived, frontend=frontend, xmods=[tmp_path])
+    assert derived['derived_type'].parent_type == BasicType.DEFERRED
+
+    # With enrichment we obtain the parent type from the parent module
+    derived = Module.from_source(fcode_derived, frontend=frontend, xmods=[tmp_path], definitions=[parent])
+    assert derived['derived_type'].parent_type is parent['parent_type']
