@@ -16,34 +16,35 @@ from pathlib import Path
 import click
 
 from loki import (
-    Sourcefile, Transformation, Scheduler, SchedulerConfig,
-    Frontend, as_tuple, set_excepthook, auto_post_mortem_debugger, info
+    Sourcefile, Frontend, as_tuple, set_excepthook,
+    auto_post_mortem_debugger, info
 )
+from loki.batch import Transformation, Pipeline, Scheduler, SchedulerConfig
 
 # Get generalized transformations provided by Loki
-from loki.transform import (
-    DependencyTransformation, ModuleWrapTransformation, FortranCTransformation,
-    FileWriteTransformation, HoistTemporaryArraysAnalysis, normalize_range_indexing,
-    InlineTransformation, SanitiseTransformation, RemoveCodeTransformation, Pipeline
-)
-
-# pylint: disable=wrong-import-order
-from transformations.argument_shape import (
+from loki.transformations.argument_shape import (
     ArgumentArrayShapeAnalysis, ExplicitArgumentArrayShapeTransformation
 )
-from transformations.data_offload import (
+from loki.transformations.array_indexing import normalize_range_indexing
+from loki.transformations.build_system import (
+    DependencyTransformation, ModuleWrapTransformation, FileWriteTransformation
+)
+from loki.transformations.data_offload import (
     DataOffloadTransformation, GlobalVariableAnalysis, GlobalVarOffloadTransformation
 )
-from transformations.derived_types import DerivedTypeArgumentsTransformation
-from transformations.drhook import DrHookTransformation
-from transformations.pool_allocator import TemporariesPoolAllocatorTransformation
-from transformations.single_column_claw import ExtractSCATransformation, CLAWTransformation
-from transformations.single_column_coalesced import (
-    SCCVectorPipeline, SCCHoistPipeline, SCCStackPipeline
-)
-from transformations.scc_cuf import (
-    HoistTemporaryArraysDeviceAllocatableTransformation
+from loki.transformations.transform_derived_types import DerivedTypeArgumentsTransformation
+from loki.transformations.drhook import DrHookTransformation
+from loki.transformations.hoist_variables import HoistTemporaryArraysAnalysis
+from loki.transformations.inline import InlineTransformation
+from loki.transformations.pool_allocator import TemporariesPoolAllocatorTransformation
+from loki.transformations.remove_code import RemoveCodeTransformation
+from loki.transformations.sanitise import SanitiseTransformation
+from loki.transformations.single_column import (
+    ExtractSCATransformation, CLAWTransformation, SCCVectorPipeline,
+    SCCHoistPipeline, SCCStackPipeline, SCCRawStackPipeline,
+    HoistTemporaryArraysDeviceAllocatableTransformation,
 )
+from loki.transformations.transpile import FortranCTransformation
 
 
 class IdemTransformation(Transformation):
@@ -69,7 +70,7 @@ def cli(debug):
 @click.option('--mode', '-m', default='idem',
               type=click.Choice(
                   ['idem', "c", 'idem-stack', 'sca', 'claw', 'scc', 'scc-hoist', 'scc-stack',
-                   'cuf-parametrise', 'cuf-hoist', 'cuf-dynamic']
+                   'cuf-parametrise', 'cuf-hoist', 'cuf-dynamic', 'scc-raw-stack']
               ),
               help='Transformation mode, selecting which code transformations to apply.')
 @click.option('--config', default=None, type=click.Path(),
@@ -182,6 +183,8 @@ def convert(
 
         scheduler.process( config.pipelines[mode] )
 
+        mode = mode.replace('-', '_')  # Sanitize mode string
+
         # Write out all modified source files into the build directory
         file_write_trafo = FileWriteTransformation(builddir=build, mode=mode)
         scheduler.process(transformation=file_write_trafo)
@@ -206,7 +209,7 @@ def convert(
     remove_code_trafo = scheduler.config.transformations.get('RemoveCodeTransformation', None)
     if not remove_code_trafo:
         remove_code_trafo = RemoveCodeTransformation(
-            remove_marked_regions=True, remove_dead_code=False,
+            remove_marked_regions=True, remove_dead_code=False, kernel_only=True,
             call_names=('ABOR1', 'DR_HOOK'), intrinsic_names=('WRITE(NULOUT',)
         )
     scheduler.process(transformation=remove_code_trafo)
@@ -279,30 +282,47 @@ def convert(
         scheduler.process( pipeline )
 
     if mode == 'scc':
-        pipeline = SCCVectorPipeline(
-            horizontal=horizontal,
-            block_dim=block_dim, directive=directive,
-            trim_vector_sections=trim_vector_sections
-        )
+        pipeline = scheduler.config.transformations.get('scc', None)
+        if not pipeline:
+            pipeline = SCCVectorPipeline(
+                horizontal=horizontal,
+                block_dim=block_dim, directive=directive,
+                trim_vector_sections=trim_vector_sections
+            )
         scheduler.process( pipeline )
 
     if mode == 'scc-hoist':
-        pipeline = SCCHoistPipeline(
-            horizontal=horizontal,
-            block_dim=block_dim, directive=directive,
-            dim_vars=(vertical.size,) if vertical else None,
-            trim_vector_sections=trim_vector_sections
-        )
+        pipeline = scheduler.config.transformations.get('scc-hoist', None)
+        if not pipeline:
+            pipeline = SCCHoistPipeline(
+                horizontal=horizontal,
+                block_dim=block_dim, directive=directive,
+                dim_vars=(vertical.size,) if vertical else None,
+                trim_vector_sections=trim_vector_sections
+            )
         scheduler.process( pipeline )
 
     if mode == 'scc-stack':
-        pipeline = SCCStackPipeline(
-            horizontal=horizontal,
-            block_dim=block_dim, directive=directive,
-            check_bounds=False,
-            trim_vector_sections=trim_vector_sections )
+        pipeline = scheduler.config.transformations.get('scc-stack', None)
+        if not pipeline:
+            pipeline = SCCStackPipeline(
+                horizontal=horizontal,
+                block_dim=block_dim, directive=directive,
+                check_bounds=False,
+                trim_vector_sections=trim_vector_sections
+            )
         scheduler.process( pipeline )
 
+    if mode == 'scc-raw-stack':
+        pipeline = scheduler.config.transformations.get('scc-raw-stack', None)
+        if not pipeline:
+            pipeline = SCCStackPipeline(
+                horizontal=horizontal,
+                block_dim=block_dim, directive=directive,
+                check_bounds=False,
+                trim_vector_sections=trim_vector_sections,
+            )
+        scheduler.process( pipeline )
 
     if mode in ['cuf-parametrise', 'cuf-hoist', 'cuf-dynamic']:
         # These transformations requires complex constructor arguments,
